# std lib
import math
import random

# pyglet
import pyglet
from pyglet.gl import *
from pyglet.window.key import *

# cocos
import cocos
from cocos.director import director
from cocos.sprite import Sprite
from cocos import euclid
from cocos.actions.interval_actions import MoveBy, ScaleBy, JumpBy, RotateBy, Delay, Accelerate
from cocos.actions.instant_actions import Hide, CallFunc
from cocos.actions.base_actions import Reverse, Repeat
from cocos.text import Label
#from cocos import path

# pymunk
#import pymunk as pm

# locals
#from primitives import *
#from state import state
#import levels
import soundex
#import HUD
#import gameover

COLL_TYPE_IGNORE, COLL_TYPE_HEAD, COLL_TYPE_BODY, COLL_TYPE_TAIL, COLL_TYPE_GOAL = range(5)

class BackgroundLayer(cocos.layer.Layer):
    def __init__(self):
        super(BackgroundLayer, self).__init__()
        grass = Sprite('grass_1_4.png', position=(1024/2, 768/2))
        self.add(grass)

class Animal(object):
    def __init__(self, x, y, direction, male, age=0):
        self.age = age
        self.male = male
        self._calculate_step()

        scale = self._calculate_scale(y)
        self._set_sprites(x, y, scale)
        if direction == 0:
            self.direction = -1
            self.sprite_l.schedule_interval(self._move_chicken, 2)
        else:
            self.direction = 1
            self.sprite_r.schedule_interval(self._move_chicken, 2)

    def _calculate_scale(self, y):
        return (1024-y+500)/float(1024-80+500)

    def _set_sprites(self, x, y, scale):
        img = self._get_image_name()
        self.head = None
        if self.age > 30 and not self.male: # hen
            body = Sprite('hen_body_r.png', position=(x, y), scale=scale)
            head = Sprite('hen_head_r.png', position=(2, 18), anchor=(0, 30), scale=scale)
            self.head = head
            body.add(head)
            self.sprite_r = body
            self.sprite_l = Sprite(img+'.png', position=(x, y), scale=scale)
        else:
            self.sprite_r = Sprite(img+'_r.png', position=(x, y), scale=scale)
            self.sprite_l = Sprite(img+'.png', position=(x, y), scale=scale)

    def _get_image_name(self):
        if self.age < 30: # chicken
            return "chick_3_1"
        else: # hen/rooster
            if self.male:
                return "rooster_3_2"
            else:
                return "hen_3_3"
        assert False

    def _calculate_step(self):
        if self.age < 10:   # egg
            self.step = 0
        elif self.age < 30: # chicken
            self.step = 20
        elif self.age < 80: # hen/rooster
            if self.male:
                self.step = 50
            else:
                self.step = 40
        elif self.age < 100: # very old
            self.step = 0
        else:                # dead
            self.step = 0

    def _do_bow(self):
        if self.head:
            r = RotateBy(70, 0.5)
            a = r + Reverse(r)
            self.head.do(a)

    def _move_chicken(self, dt):
        if self.age >= 100:
            return
        if self.direction == -1:
            sprite = self.sprite_l
            sprite_2 = self.sprite_r
        else:
            sprite = self.sprite_r
            sprite_2 = self.sprite_l
        if sprite.x > 80 and sprite.x < 1024-80:
            a = Accelerate(MoveBy((self.direction*random.random()*50, 0), 1), 2)
            if self.direction == 1:
                a = a + CallFunc(self._do_bow)
            sprite.do(a)
            sprite.unschedule(self._move_chicken)
            sprite.schedule_interval(self._move_chicken, 1+random.random()*2)
        else:
            self.direction = -1 * self.direction
            parent = sprite.parent
            sprite.unschedule(self._move_chicken)
            sprite_2.x = sprite.x
            sprite_2.y = sprite.y
            parent.remove(sprite)
            parent.add_animal(sprite_2)
            step_x = self.direction * (random.random()+0.5) * self.step
            sprite_2.do(Accelerate(MoveBy((step_x, 0), 1), 2))
            sprite_2.schedule_interval(self._move_chicken, 1+random.random()*2)


    def get_sprite(self):
        if self.direction == -1:
            return self.sprite_l
        else:
            return self.sprite_r

    def update(self, c):
        self.age += 1
        if self.age == 30:
            sprite = self.get_sprite()
            parent = sprite.parent
            img = self._get_image_name()
            scale = self._calculate_scale(sprite.y)
            self.sprite_r = Sprite(img+'_r.png', position=(sprite.x, sprite.y), scale=scale)
            self.sprite_l = Sprite(img+'.png', position=(sprite.x, sprite.y), scale=scale)
            parent.remove(sprite)
            sprite = self.get_sprite()
            parent.add(sprite)
        elif self.age == 100:
            self.kill()

    def kill(self):
        if self.direction == -1:
            sprite = self.sprite_l
            r = RotateBy(180, 1)
        else:
            sprite = self.sprite_r
            r = RotateBy(-180, 1)
        a = JumpBy(height=100, duration=1) | r
        a = a + Delay(0.5) + Hide() + CallFunc(self._destroy)
        sprite.do(a)

    def _destroy(self):
        sprite = self.get_sprite()
        parent = sprite.parent
        parent.remove(sprite)
        for cb in sprite.scheduled_calls + sprite.scheduled_interval_calls:
            sprite.unschedule(cb)
        

class CounterLabel(Label):
    def __init__(self, collection_name, num, x, y):
        text = "%s: %d" % (collection_name, num)
        super(CounterLabel, self).__init__(text, position=(x, y), bold=True)


class Feeder(object):
    def __init__(self, farm):
        self.farm = farm
        self.feeder_btn = Sprite('feeder_1.png', position=(515, 30))
        self.grain = 80
        self.grain_lbl = Label("Grain: %d" % self.grain, position=(480, 60), bold=True)
        self.last_feeding_time = 0

    def is_inside(self, x, y):
        if x > 481 and x < 547 and y > 10 and y < 50:
            return True
        return False

    def clicked(self):
        self.grain -= 10
        self.last_feeding_time = self.farm.clock
        self.grain_lbl.element.text = "Grain: %d" % self.grain
        if random.randint(0, 1) == 0:
            d = 1
        else:
            d = -1
        m = MoveBy((d*20, 50), 0.3) + MoveBy((d*30, 50), 0.3) + MoveBy((d*40, 50), 0.3)
        m = Accelerate(m, 1.4)
        self.feeder_btn.do(m + Reverse(m))


class CtrlLayer(cocos.layer.Layer):
    is_event_handler = True
    def __init__(self):
        super(CtrlLayer, self).__init__()
        #director.interpreter_locals = {'hen': hen, 'bg': self}
        self.animals = self._generate_animals(10, (30, 80), False) # hens
        self.animals.extend(self._generate_animals(5, (30, 80), True)) # roosters
        self.animals.extend(self._generate_animals(7, (10, 30), False)) # chicks - female
        self.animals.extend(self._generate_animals(3, (10, 30), True)) # chicks - male

        # control panel
        self.hens_cnt = CounterLabel("Hens", 0, 10, 60)
        self.add(self.hens_cnt)
        self.roosters_cnt = CounterLabel("Roosters", 0, 10, 30)
        self.add(self.roosters_cnt)
        self.chickens_cnt = CounterLabel("Chickens", 0, 100, 60)
        self.add(self.chickens_cnt)
        self.chicks_cnt = CounterLabel("Chicks", 0, 120, 30)
        self.add(self.chicks_cnt)
        self.eggs_cnt = CounterLabel("Eggs", 0, 330, 30)
        self.add(self.eggs_cnt)
        self._count_animals()

        self.feeder = Feeder(self)

        cash_cnt = CounterLabel("Cash", 45, 230, 30)
        self.add(cash_cnt)
        #grain_cnt = CounterLabel("Grain", 80, 230, 60)
        #self.add(grain_cnt)
        self.add(self.feeder.grain_lbl)

        conds_lbl = Label("Conditions: well", (330, 60), bold=True)
        self.add(conds_lbl)

        self.add(self.feeder.feeder_btn, z=10000)

        self.clock_lbl = Label("Time", (700, 60), bold=True)
        self.add(self.clock_lbl)
        self.clock = 0
        self._update_clock()

        self.schedule_interval(self._update, 5)

    def _update_clock(self):
        days = self.clock / 12
        hour = 8 + self.clock % 12
        self.clock_lbl.element.text = "Day %d, hour %d:00" % (days+1, hour) 

    def _count_animals(self):
        hens_num = 0
        roosters_num = 0
        chickens_num = 0
        eggs_num = 0
        for a in self.animals:
            if a.age < 10:
                eggs_num += 1
            elif a.age < 30:
                chickens_num += 1
            else:
                if a.male:
                    roosters_num += 1
                else:
                    hens_num += 1
        self.hens_cnt.element.text = "Hens: %d" % hens_num
        self.roosters_cnt.element.text = "Roosters: %d" % roosters_num
        self.chicks_cnt.element.text = "Chicks: %d" % chickens_num
        self.eggs_cnt.element.text = "Eggs: %d" % eggs_num

    def on_mouse_press (self, x, y, buttons, modifiers):
        print x, y
        if self.feeder.is_inside(x, y):
            self.feeder.clicked()

    def add_animal(self, hen):
        self.add(hen, z=1024-hen.y)

    def _generate_animals(self, num, age_range, male):
        l = []
        for i in range(num):
            y = (768-670) + random.randint(80, 670-160)
            a = Animal(random.randint(80, 1024-80),
                       y,
                       random.randint(0, 1),      # direction
                       male,
                       random.randint(*age_range))    # age
            l.append(a)
            self.add_animal(a.get_sprite())
        return l
    
    def _update(self, dt):
        self.clock += 1
        self._update_clock()
        
        no_food_period = self.clock - self.feeder.last_feeding_time
        if no_food_period < 12:
            ratio = 0
        elif no_food_period < 24:
            ratio = 0.3
        elif no_food_period < 36:
            ratio = 0.6
        else:
            ratio = 1.1
        for a in self.animals:
            r = random.random()
            if r < ratio:
                a.kill()
                self.animals.remove(a)

        eggs = []        
        for a in self.animals:
            a.update(self.clock)
            if a.age % 12 and a.age > 30 and a.male == False and random.random() > 0.95:
                male = (int(round(random.random()*0.8)) == 1)
                eggs.append(self._generate_animals(1, (0, 0), male)[0])
            if a.age == 100:
                self.animals.remove(a)
        self.animals.extend(eggs)
        self._count_animals()

def get_game_scene():
    #state.reset()

    s = cocos.scene.Scene()
    s.add(BackgroundLayer(), z=0)
    s.add(CtrlLayer(), z=1)
    #s.add( HUD.BackgroundLayer(), z=-1 )
    #hud = HUD.HUD()
    #s.add( hud, z=1, name='hud' )
    #gameModel = GameLayer(demo=False, start_level=state.start_level)
    #s.add( gameModel, z=0 )
    #s.add( ControlLayer( gameModel), z=0, name='ctrl' )
    #hud.show_message( 'Lvl %d: %s' % (state.level_idx, state.level.title) )
    return s
